@@ -1,5 +1,5 @@
 /* libctl: flexible Guile-based control files for scientific software 
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2006, Steven G. Johnson
+ * Copyright (C) 1998-2009, Steven G. Johnson
  *
  * This file may be used without restriction.  It is in the public
  * domain, and is NOT restricted by the terms of any GNU license.
@@ -151,6 +151,13 @@ extern void ctl_stop_hook(void);
 extern void ctl_export_hook(void);
 #endif
 
+extern SCM nlopt_minimize_scm(SCM algorithm_scm,
+		       SCM f_scm,
+                       SCM lb_scm, SCM ub_scm, SCM x_scm,
+		       SCM minf_max_scm, SCM ftol_rel_scm, SCM ftol_abs_scm,
+		       SCM rest);
+
+
 /* Main program.  Start up Guile, declare functions, load any
    scripts passed on the command-line, and drop into interactive
    mode if read-input-vars was never called. */
@@ -174,6 +181,12 @@ void main_entry(int argc, char *argv[])
   /* Export the subplex minimization routine: */
   gh_new_procedure ("subplex", (SCM (*)(void)) subplex_scm, 7, 0, 0);
 
+#ifdef HAVE_NLOPT
+  /* Export the nlopt minimization routine, if available: */
+  gh_new_procedure ("nlopt-minimize", (SCM (*)(void)) nlopt_minimize_scm, 
+		    8, 0, 1);
+#endif
+
   /* Export the adaptive integration routines: */
   gh_new_procedure ("adaptive-integration", 
 		    (SCM (*)(void)) adaptive_integration_scm, 6, 0, 0);
