@@ -293,7 +293,7 @@ static num evalRR0_0fs(integrand f, void *fdata, unsigned dim, double *p, const
      return sum;
 }
 
-static double evalR0_0fs4d(integrand f, void *fdata, unsigned dim, double *p, const double *c, num *sum0_, const double *r1, num *sum1_, const double *r2, num *sum2_)
+static unsigned evalR0_0fs4d(integrand f, void *fdata, unsigned dim, double *p, const double *c, num *sum0_, const double *r1, num *sum1_, const double *r2, num *sum2_)
 {
      double maxdiff = 0;
      unsigned i, dimDiffMax = 0;
@@ -927,7 +927,7 @@ int main(int argc, char **argv)
      xmax = (double *) malloc(dim * sizeof(double));
      for (i = 0; i < dim; ++i) {
 	  xmin[i] = 0;
-	  xmax[i] = 1 + (which_integrand >= 2 ? 0 : 0.4 * sin(i));
+	  xmax[i] = 1 + (which_integrand >= 1 ? 0 : 0.4 * sin(i));
      }
 
      printf("%u-dim integral, tolerance = %g, integrand = %d\n",
