@@ -1,5 +1,5 @@
 /* libctl: flexible Guile-based control files for scientific software 
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2006, Steven G. Johnson
+ * Copyright (C) 1998-2009, Steven G. Johnson
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -959,7 +959,7 @@ static number compute_dot_cross(vector3 a, vector3 b, vector3 c)
 
    Requires that geometry_lattice global has been initialized,
    etcetera.  */
-static void get_bounding_box(geometric_object o, geom_box *box)
+void geom_get_bounding_box(geometric_object o, geom_box *box)
 {
      geom_fix_object(o);
 
@@ -1099,7 +1099,7 @@ static void get_bounding_box(geometric_object o, geom_box *box)
 		   ->component_objects.items;
 	      for (i = 0; i < n; ++i) {
 		   geom_box boxi;
-		   get_bounding_box(os[i], &boxi);
+		   geom_get_bounding_box(os[i], &boxi);
 		   geom_box_shift(&boxi, o.center);
 		   geom_box_union(box, box, &boxi);
 	      }
@@ -1109,8 +1109,10 @@ static void get_bounding_box(geometric_object o, geom_box *box)
 }
 
 /**************************************************************************/
-/* compute the fraction of a box's volume (or area/length in 2d/1d) that
-   overlaps an object */
+/* Compute the fraction of a box's volume (or area/length in 2d/1d) that
+   overlaps an object.   Instead of a box, we also allow an ellipsoid
+   inscribed inside the box (or a skewed ellipsoid if the box is not
+   orthogonal). */
 
 typedef struct {
      geometric_object o;
@@ -1118,7 +1120,10 @@ typedef struct {
      int pdim[2]; /* the (up to two) integration directions */
      double scx[2]; /* scale factor (e.g. sign flip) for x coordinates */
      unsigned dim;
-     double a0, b0;
+     double a0, b0; /* box limits along analytic direction */
+     int is_ellipsoid; /* 0 for box, 1 for ellipsoid */
+     double winv[2], c[2]; /* ellipsoid width-inverses/centers in int. dirs */
+     double w0, c0; /* width/center along analytic direction */
 } overlap_data;
 
 static double overlap_integrand(integer ndim, number *x, void *data_)
@@ -1127,6 +1132,8 @@ static double overlap_integrand(integer ndim, number *x, void *data_)
      double s[2];
      const double *scx = data->scx;
      vector3 p = data->p;
+     double a0 = data->a0, b0 = data->b0;
+     double scale_result = 1.0;
 
      if (ndim > 0) {
 	  switch (data->pdim[0]) {
@@ -1143,17 +1150,42 @@ static double overlap_integrand(integer ndim, number *x, void *data_)
 	  }
      }
 
+     if (data->is_ellipsoid && ndim > 0) {
+	  /* compute width of ellipsoid at this point, along the
+	     analytic-intersection direction */
+	  double dx = (x[0] - data->c[0]) * data->winv[0];
+	  double w = 1.0 - dx * dx;
+	  if (ndim > 1) { /* rescale 2nd dimension to stay inside ellipsoid */
+	       double x1;
+	       if (w < 0) return 0.0; /* outside the ellipsoid */
+	       scale_result = sqrt(w);
+	       x1 = data->c[1] + (x[1] - data->c[1]) * scale_result;
+	       switch (data->pdim[1]) {
+		   case 0: p.x = scx[1] * x1; break;
+		   case 1: p.y = scx[1] * x1; break;
+		   case 2: p.z = scx[1] * x1; break;
+               }
+	       dx = (x1 - data->c[1]) * data->winv[1];
+	       w -= dx * dx;
+	  }
+	  if (w < 0) return 0.0; /* outside the ellipsoid */
+	  w = data->w0 * sqrt(w);
+	  a0 = data->c0 - w; b0 = data->c0 + w;
+     }
+
      if (2 == intersect_line_with_object(p, data->dir, data->o, s)) {
 	  double ds = (s[0] < s[1]
-		       ? MIN(s[1],data->b0) - MAX(s[0],data->a0)
-		       : MIN(s[0],data->b0) - MAX(s[1],data->a0));
-	  return (ds > 0 ? ds : 0.0);
+		       ? MIN(s[1],b0) - MAX(s[0],a0)
+		       : MIN(s[0],b0) - MAX(s[1],a0));
+	  return (ds > 0 ? ds * scale_result : 0.0);
      }
      return 0.0;
 }
 
-number box_overlap_with_object(geom_box b, geometric_object o,
-			       number tol, integer maxeval)
+#define K_PI 3.14159265358979323846
+
+number overlap_with_object(geom_box b, int is_ellipsoid, geometric_object o,
+			   number tol, integer maxeval)
 {
      overlap_data data;
      int empty_x = b.low.x == b.high.x;
@@ -1164,11 +1196,11 @@ number box_overlap_with_object(geom_box b, geometric_object o,
 		  (empty_z ? 1 : b.high.z - b.low.z));
      vector3 ex = {1,0,0}, ey = {0,1,0}, ez = {0,0,1};
      geom_box bb;
-     double xmin[2], xmax[2], esterr;
+     double xmin[2] = {0,0}, xmax[2] = {0,0}, esterr;
      int errflag;
      unsigned i;
 
-     get_bounding_box(o, &bb);
+     geom_get_bounding_box(o, &bb);
      geom_box_intersection(&bb, &b, &bb);
      if (bb.low.x > bb.high.x || bb.low.y > bb.high.y || bb.low.z > bb.high.z
 	 || (!empty_x && bb.low.x == bb.high.x)
@@ -1176,6 +1208,9 @@ number box_overlap_with_object(geom_box b, geometric_object o,
 	 || (!empty_z && bb.low.z == bb.high.z))
 	  return 0.0;
 
+     data.winv[0] = data.winv[1] = data.w0 = 1.0;
+     data.c[0] = data.c[1] = data.c0 = 0;
+
      data.o = o;
      data.p.x = data.p.y = data.p.z = 0;
      data.dim = 0;
@@ -1183,14 +1218,20 @@ number box_overlap_with_object(geom_box b, geometric_object o,
 	  data.dir = ex;
 	  data.a0 = bb.low.x;
 	  data.b0 = bb.high.x;
+	  data.w0 = 0.5 * (b.high.x - b.low.x);
+	  data.c0 = 0.5 * (b.high.x + b.low.x);
 	  if (!empty_y) {
 	       xmin[data.dim] = bb.low.y;
 	       xmax[data.dim] = bb.high.y;
+	       data.winv[data.dim] = 2.0 / (b.high.y - b.low.y);
+	       data.c[data.dim] = 0.5 * (b.high.y + b.low.y);
 	       data.pdim[data.dim++] = 1;
 	  }
 	  if (!empty_z) {
 	       xmin[data.dim] = bb.low.z;
 	       xmax[data.dim] = bb.high.z;
+	       data.winv[data.dim] = 2.0 / (b.high.z - b.low.z);
+	       data.c[data.dim] = 0.5 * (b.high.z + b.low.z);
 	       data.pdim[data.dim++] = 2;
 	  }
      }
@@ -1198,14 +1239,20 @@ number box_overlap_with_object(geom_box b, geometric_object o,
 	  data.dir = ey;
 	  data.a0 = bb.low.y;
 	  data.b0 = bb.high.y;
+	  data.w0 = 0.5 * (b.high.y - b.low.y);
+	  data.c0 = 0.5 * (b.high.y + b.low.y);
 	  if (!empty_x) {
 	       xmin[data.dim] = bb.low.x;
 	       xmax[data.dim] = bb.high.x;
+	       data.winv[data.dim] = 2.0 / (b.high.x - b.low.x);
+	       data.c[data.dim] = 0.5 * (b.high.x + b.low.x);
 	       data.pdim[data.dim++] = 0;
 	  }
 	  if (!empty_z) {
 	       xmin[data.dim] = bb.low.z;
 	       xmax[data.dim] = bb.high.z;
+	       data.winv[data.dim] = 2.0 / (b.high.z - b.low.z);
+	       data.c[data.dim] = 0.5 * (b.high.z + b.low.z);
 	       data.pdim[data.dim++] = 2;
 	  }
      }
@@ -1213,20 +1260,27 @@ number box_overlap_with_object(geom_box b, geometric_object o,
 	  data.dir = ez;
 	  data.a0 = bb.low.z;
 	  data.b0 = bb.high.z;
+	  data.w0 = 0.5 * (b.high.z - b.low.z);
+	  data.c0 = 0.5 * (b.high.z + b.low.z);
 	  if (!empty_x) {
 	       xmin[data.dim] = bb.low.x;
 	       xmax[data.dim] = bb.high.x;
+	       data.winv[data.dim] = 2.0 / (b.high.x - b.low.x);
+	       data.c[data.dim] = 0.5 * (b.high.x + b.low.x);
 	       data.pdim[data.dim++] = 0;
 	  }
 	  if (!empty_y) {
 	       xmin[data.dim] = bb.low.y;
 	       xmax[data.dim] = bb.high.y;
+	       data.winv[data.dim] = 2.0 / (b.high.y - b.low.y);
+	       data.c[data.dim] = 0.5 * (b.high.y + b.low.y);
 	       data.pdim[data.dim++] = 1;
 	  }
      }
      else
 	  return 1.0;
 
+#if 1
      /* To maintain mirror symmetries through the x/y/z axes, we flip
         the integration range whenever xmax < 0.  (This is in case
         the integration routine is not fully symmetric, which may
@@ -1237,10 +1291,21 @@ number box_overlap_with_object(geom_box b, geometric_object o,
 	       data.scx[i] = -1;
 	       xmin[i] = -xmax[i];
 	       xmax[i] = -xm;
+	       data.c[i] = -data.c[i];
 	  }
 	  else
 	       data.scx[i] = 1;
      }
+#else
+     for (i = 0; i < data.dim; ++i) data.scx[i] = 1;
+#endif
+
+     if ((data.is_ellipsoid = is_ellipsoid)) { /* data for ellipsoid calc. */
+	  if (data.dim == 1)
+	       V0 *= K_PI / 4;
+	  else if (data.dim == 2)
+	       V0 *= K_PI / 6; 
+     }
 
      return adaptive_integration(overlap_integrand, xmin, xmax, 
 				 data.dim, &data, 
@@ -1248,6 +1313,18 @@ number box_overlap_with_object(geom_box b, geometric_object o,
 				 &esterr, &errflag) / V0;
 }
 
+number box_overlap_with_object(geom_box b, geometric_object o,
+			       number tol, integer maxeval)
+{
+     return overlap_with_object(b, 0, o, tol, maxeval);
+}
+
+number ellipsoid_overlap_with_object(geom_box b, geometric_object o,
+				     number tol, integer maxeval)
+{
+     return overlap_with_object(b, 1, o, tol, maxeval);
+}
+
 number CTLIO range_overlap_with_object(vector3 low, vector3 high,
 				       geometric_object o, number tol, 
 				       integer maxeval)
@@ -1281,7 +1358,7 @@ void destroy_geom_box_tree(geom_box_tree t)
 static int object_in_box(geometric_object o, vector3 shiftby,
 			 geom_box *obj_b, const geom_box *b)
 {
-     get_bounding_box(o, obj_b);
+     geom_get_bounding_box(o, obj_b);
      geom_box_shift(obj_b, shiftby);
      return geom_boxes_intersect(obj_b, b);
 }
